@@ -1,5 +1,5 @@
 # Simple Python module for retrieving data from bitchute.
-# Copyright (C) 2020 Marcus Burkhardt
+# Copyright (C) 2022 Marcus Burkhardt
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -131,8 +131,19 @@ class Crawler():
         return page_source
     
     def process_views(self, views):
-        views = views.replace('K', '00').replace('k', '00').replace('.', '')
-        return views 
+        if "k" in views or "K" in views:
+            views = views.replace('K', '').replace('k', '')
+            if '.' not in views:
+                views = views[:-1]+'.'+views[-1:]
+            views = float(views) * 1000
+        elif "m" in views or "M" in views:
+            views = views.replace('M', '').replace('m', '')
+            if '.' in views:
+                views = float(views)
+            else:
+                views = float(views)/10
+            views = views * 1000000
+        return int(views)
 
     def search(self, query, top=100):
         '''
@@ -146,11 +157,13 @@ class Crawler():
         data: Dataframe of search results.
         '''
         url = self.search_base.format(query)
+        if isinstance(top, str) and top.lower()=='all':
+            top = None
         src = self.call(url, top=top)
         data = self.parser(src, type='video_search')
         self.reset_webdriver()
         return data   
-
+    
     def get_recommended_videos(self, type='popular'):
         '''
         Scapes recommended videos on bitchute homepage.
@@ -190,6 +203,24 @@ class Crawler():
             return None
         self.reset_webdriver()
 
+    def get_popular_videos(self):
+        videos, tags = self.get_recommended_videos(type='popular')
+        return videos
+    
+    def get_trending_videos(self):
+        videos, tags = self.get_recommended_videos(type='trending')
+        return videos
+    def get_trending_tags(self):
+        videos, tags = self.get_recommended_videos(type='trending')
+        return tags
+    def get_trending(self):
+        videos, tags = self.get_recommended_videos(type='trending')
+        return videos, tags
+    
+    def get_all_videos(self):
+        videos, tags = self.get_recommended_videos(type='all')
+        return videos
+
     def get_recommended_channels(self, extended=True):
         '''
         Scapes recommended channels on bitchute homepage.
@@ -257,8 +288,8 @@ class Crawler():
             videos = pd.DataFrame()
             for channel_id in (tqdm(channel_ids) if not self.verbose else channel_ids):
                 about_tmp, videos_tmp = self._get_channel(channel_id, get_channel_about=get_channel_about, get_channel_videos=get_channel_videos)
-                abouts = abouts.append(about_tmp)
-                videos = videos.append(videos_tmp)
+                abouts = pd.concat([abouts, about_tmp])
+                videos = pd.concat([videos, videos_tmp])
             self.reset_webdriver()
             return abouts, videos
         else:
@@ -278,7 +309,7 @@ class Crawler():
         '''
 
         video_url = self.video_base.format(video_id)
-        src = self.call(video_url)
+        src = self.call(video_url, scroll=False)
         video_data = self.parser(src, type='video')
         return video_data
 
@@ -305,7 +336,8 @@ class Crawler():
             for video_id in (tqdm(video_ids) if not self.verbose else video_ids):
                 try:
                     video_tmp = self._get_video(video_id)                
-                    video_data = video_data.append(video_tmp)
+                    if video_tmp is not None:
+                        video_data = pd.concat([video_data, video_tmp])
                 except:
                     print('Failed for video with id {}'.format(video_id))
                     self.reset_webdriver()
@@ -351,8 +383,9 @@ class Crawler():
             video_data = pd.DataFrame()
             for hashtag in (tqdm(hashtags) if not self.verbose else hashtags):
                 video_tmp = self._get_hashtag(hashtag)
-                video_tmp['hashtag'] = hashtag             
-                video_data = video_data.append(video_tmp)
+                if video_tmp is not None:
+                    video_tmp['hashtag'] = hashtag             
+                    video_data = video_data.pd.concat([video_tmp])
             self.reset_webdriver()
             return video_data
         else:
@@ -361,6 +394,11 @@ class Crawler():
 
     def parser(self, src, type=None, kind=None, extended=False):
         scrape_time = str(int(datetime.utcnow().timestamp()))
+        
+        soup = BeautifulSoup(src, 'html.parser')
+        if soup.find('h1') and ("404 - Page not found" in soup.find('h1').text or "404 - PAGE NOT FOUND" in soup.find('h1').text):
+            return None
+
         if not type:
             raise 'A parse type needs to be passed.'
         
@@ -547,6 +585,7 @@ class Crawler():
             return videos, tags
 
         elif type == 'channel_about':
+            uid = None
             id_ = None
             title = None
             owner = None
@@ -562,9 +601,11 @@ class Crawler():
 
             soup = BeautifulSoup(src, 'html.parser')            
             if soup.find('link', id='canonical'):
-                id_ = soup.find('link', id='canonical').get('href').split('/')[-2]
+                uid = soup.find('link', id='canonical').get('href').split('/')[-2]
             if soup.find(class_='name'):
                 title = soup.find(class_='name').text.strip('\n').strip()
+                if soup.find(class_='name').find('a'):
+                    id_ = soup.find(class_='name').find('a').get('href').strip('/').split('/')[1]
             if soup.find(class_='owner'):
                 owner = soup.find(class_='owner').text.strip('\n').strip()
                 owner_link = soup.find(class_='owner').find('a').get('href')
@@ -590,8 +631,8 @@ class Crawler():
                     else:
                         created_at = elem.text.strip('\n').strip()
                         pass
-            data = [id_, title, social_links, description, description_links, video_count, subscriber_count, view_count, created_at, category, social_links, owner, owner_link, scrape_time]
-            columns = ['id', 'title', 'social_links', 'description', 'description_links', 'video_count', 'subscriber_count', 'view_count', 'created_at', 'category', 'social_links', 'owner', 'owner_link', 'scrape_time']
+            data = [uid, id_, title, description, description_links, video_count, subscriber_count, view_count, created_at, category, social_links, owner, owner_link, scrape_time]
+            columns = ['uid', 'id', 'title', 'description', 'description_links', 'video_count', 'subscriber_count', 'view_count', 'created_at', 'category', 'social_links', 'owner', 'owner_link', 'scrape_time']
             data = pd.DataFrame([data], columns=columns)
             return data
 
@@ -628,7 +669,7 @@ class Crawler():
                     else:
                         duration = None
                     if video.find(class_='channel-videos-details'):
-                        created_at = str(dateutil.parser.parse( video.find(class_='channel-videos-details').text.replace('\n', '')).date())
+                        created_at = str(parser.parse( video.find(class_='channel-videos-details').text.replace('\n', '')).date())
                     else:
                         created_at = None
                     if video.find(class_='video-views'):
